/* --------------------------------------------------------------------------------
   LavA - A Runtime Platform for Application-Specific Distributed Architectures

   Copyright (C) 2009-2012  Embedded System Software Group, Computer Science 12,
   TU Dortmund, <http://ess.cs.tu-dortmund.de/EN/Research/Projects/LavA/>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program. If not, see <http://www.gnu.org/licenses/>.
   -------------------------------------------------------------------------------- */


#include <inttypes.h>
#include "main.h"
#include "io.h"


void irq_handler() {
	interrupt = *INTERRUPT_STAT;

	if ((interrupt & 0x01) != 0) {
		uart_handle_rx_irq();
	} else if ((interrupt & 0x02) != 0) {
		ipc_handle_rx_irq();
	}
}


void uart_handle_rx_irq() {
	data = *UART_RX;

	*IPC_ADDR = 0x04;
	*IPC_DATA = data;
	*OUTPORT ^= (1 << PIN1);
}


void ipc_handle_rx_irq() {
	addr = *IPC_ADDR;
	data = *IPC_DATA;

	uart_putc((char)data);
}


void uart_putc(char c) {
	while ((*UART_TX & (1 << TXRDY)) == 0) {}
	*UART_TX = c;
}


void uart_puthex(uint32_t c, uint8_t bytes) {
	if (bytes > 4) bytes = 4;

	switch (bytes) {
		case 4:
		    uart_putc("0123456789ABCDEF"[(c&0xf0000000)>>28]);
		    uart_putc("0123456789ABCDEF"[(c&0x0f000000)>>24]);
		case 3:
		    uart_putc("0123456789ABCDEF"[(c&0x00f00000)>>20]);
		    uart_putc("0123456789ABCDEF"[(c&0x000f0000)>>16]);
		case 2:
		    uart_putc("0123456789ABCDEF"[(c&0x0000f000)>>12]);
		    uart_putc("0123456789ABCDEF"[(c&0x00000f00)>>8]);
		default:
		    uart_putc("0123456789ABCDEF"[(c&0x000000f0)>>4]);
		    uart_putc("0123456789ABCDEF"[(c&0x0000000f)]);
	}
}


void uart_puts(const char *s) {
	while (*s) {
		uart_putc (*s);
		s++;
	}
}


void init(void) {
	// Interrupts
	*INTERRUPT_ENA |= (1 << IPC_IRQ) | (1 << UART_IRQ);
	*INTERRUPT_CTRL |= (1 << ENA_IRQ);
}


int main(int argc, char **argv) {

	init();

	*OUTPORT |= (1 << PIN0);
	uart_puts("Hello World!");

	while (1);

	return 0;
}
