/* --------------------------------------------------------------------------------
   LavA - A Runtime Platform for Application-Specific Distributed Architectures

   Copyright (C) 2009-2012  Embedded System Software Group, Computer Science 12,
   TU Dortmund, <http://ess.cs.tu-dortmund.de/EN/Research/Projects/LavA/>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program. If not, see <http://www.gnu.org/licenses/>.
   -------------------------------------------------------------------------------- */


#include "main.h"
#include "io.h"


void InterruptServiceRoutine(unsigned int status) {
	interrupt = *INTERRUPT_STAT;

	if ((interrupt & 0x01) != 0) {
		timer1_handle_irq();
	} else if ((interrupt & 0x02) != 0) {
		timer2_handle_irq();
	} else if ((interrupt & 0x04) != 0) {
		ipc_handle_rx_irq();
	}
}


void timer1_handle_irq() {
	*TIMER1_CNT1 |= (1 << ACK_IRQ_TMR) | (1 << ENA_TMR) | (1 << RST_TMR);

	*OUTPORT ^= (1 << PIN1);
}


void timer2_handle_irq() {
	*TIMER2_CNT1 |= (1 << ACK_IRQ_TMR) | (1 << ENA_TMR) | (1 << RST_TMR);

	*IPC_ADDR = 0x01;
	*IPC_DATA = (int)'.';
}


void ipc_handle_rx_irq() {
	addr = *IPC_ADDR;
	data = *IPC_DATA;
}


void init(void) {
	// Interrupts aktivieren
	*INTERRUPT_ENA |= (1 << IPC_IRQ) | (1 << TMR2_IRQ) | (1 << TMR1_IRQ);
	*INTERRUPT_CTRL |= (1 << ENA_IRQ);
	AsmInterruptEnable(1);

	// Timer initialisieren
	*TIMER1_COMP1 = 0x00FFFFFF;
	*TIMER1_CNT1 |= (1 << ENA_TMR) | (1 << RST_TMR);

	*TIMER2_COMP1 = 0xFFFFFFFF;
	*TIMER2_COMP2 = 0x00000001;
	*TIMER2_CNT1 |= (1 << ENA_TMR) | (1 << RST_TMR);
}


int main(int argc, char **argv) {

	init();

	*OUTPORT |= (1 << PIN0);

	while(1);

	return 0;
}
